
importData = function(ff) { #THIS FUNCTION TAKES ONLY 1 ARGUMENT (a variable being a name of one of the data files inside the selected case-folder).
 library(euroformix)
 ln = euroformix::getKit("GlobalFiler","Marker")  #considered loci
 LOD = 70 #Detection threshold set to 70rfu by default.

 ef = "evids.csv"
 rf = "refs.csv"
 gf = "freqs.csv" #M Number of reference simulations given below
 bigref <- bigevid <- meta <- numeric()

 #Read file:
 tab = euroformix::tableReader(ff) #read file
 cn <- colnames(tab) #get column names

############################
#CONSIDER EVIDENCE PROFILES#
############################
if(grepl(ef,ff)) {
   #get column indices
   lind = grep("marker",tolower(cn),fixed=TRUE) #locus col-ind
   sind = grep("sample",tolower(cn),fixed=TRUE) #sample col-ind
   A_ind = grep("allele",tolower(cn),fixed=TRUE) #allele col-ind
   H_ind = grep("height",tolower(cn),fixed=TRUE) #height col-ind
   sn = unique(as.character(tab[,sind])) #sample names
 
   bigmat <- numeric()
   for(s in sn) { #for each ID
    for(loc in ln) { #for each locus
     newline <- c(s, loc)
     xind = tab[,sind]==s & toupper(tab[,lind])==toupper(loc) #get index in tab for given sample and locus
     if(!any(xind)) {
       newline <- c(newline,"","")
     } else {
       hv <-  as.character(tab[xind,H_ind]) #get height vector
       keep = !is.na(hv) & hv!="NA"
       keep[which(as.numeric(hv)<LOD)]  = FALSE
       hv = hv[keep] #get those to keep
       av <- as.character(tab[xind,A_ind])[keep] #get allele vector
       if(length(av)!=length(hv)) {
 		    stop(paste0("Allele length wrong for sample ",s," at locus ",loc))
       }
       newline <- c(newline,paste0(av,collapse="/")) #collapse alleles (THIS IS REQUIRED FORMAT FOR CASESOLVER)
       newline <- c(newline,paste0(hv,collapse="/")) #collapse peak heights (THIS IS REQUIRED FORMAT FOR CASESOLVER)
     }
     bigmat <- rbind(bigmat , newline ) #add new line
   } #end for each loci
  } #end for each sample
  bigevid = bigmat
  colnames(bigevid) <- c("SampleName","Markers","Alleles","Heights")
# bigevid[,1] = sapply(strsplit(bigevid[,1],"-"),function(x) x[4])

 #METADATA:
 agg = aggregate( bigevid[,3]!="", by = list(bigevid[,1]),sum)
 Negative = agg$Gr[agg$x == 0] #extract negative samples
 Few = agg$Gr[agg$x>0 & agg$x<7 ] #extract samples with few 
 meta = list('Negative samples'=cbind(Negative),'Samples with less than 7 markers'=cbind(Few))
 bigevid = bigevid[!bigevid[,1]%in%c(Negative,Few),] #remove samples with few alleles
 if(!all(sapply(strsplit(bigevid[,3],"/"),length)==sapply(strsplit(bigevid[,4],"/"),length))) print("WARNING: THE NUMBER OF ALLELES WAS NOT THE SAME AS NUMBER OF PEAK HEIGHTS!")
}#end if EVID

#############################
#CONSIDER REFERENCE PROFILES#
#############################
if(grepl(rf,ff)) {
 lind = grep("marker",tolower(cn),fixed=TRUE) #locus col-ind
 sind = grep("sample",tolower(cn),fixed=TRUE) #sample col-ind
 A_ind = grep("allele",tolower(cn),fixed=TRUE) #allele col-ind
 sn = unique(as.character(tab[,sind])) #sample names

 bigmat <- numeric()
 for(s in sn) { #for each ID
  for(loc in ln) { #for each locus
   newline <- c(s, loc)
   xind = tab[,sind]==s & toupper(tab[,lind])==toupper(loc) #get index in tab for given sample and locus
   if(!any(xind)) {
      newline <- c(newline,"")
   } else {
    av <- as.character(tab[xind,A_ind]) #get allele vector
    av <- av[!is.na(av) & av!="NA" & av!=""]
    if(length(av)==1 && !loc%in%c("Yindel","DYS391")) av = rep(av,2) #hom. given twice for aSTRs
    newline <- c(newline,paste0(av,collapse="/")) #collapse alleles (THIS IS REQUIRED FORMAT FOR CASESOLVER)
   }
   bigmat <- rbind(bigmat , newline ) #add new line
  } #end for each loci
 } #end for each sample
 bigref <- bigmat 
 colnames(bigref) <- c("SampleName","Markers","Alleles")
 meta = NULL #No meta data in case of references
} #end if references

##############################
##GENERATE REFERENCE PROFILES#
##############################
if(grepl(gf,ff)) {
 set.seed(1) #Same seed as used in paper
 M = 1e6  #M Number of generated references. SPECIFY HERE!
 rn = sprintf(paste0("g%0",nchar(M),"d"), 1:M)  #names of simulated refs

 locs <- toupper(cn[-1])
 minF = min(tab,na.rm=TRUE) #aquire minimum freq
 popFreq <- list()
 for(i in 1:length(locs)) {
  freqs <- tab[,i+1]
  av = tab[!is.na(freqs),1] #allele names
  fv = tab[!is.na(freqs),i+1] #freqs
  if(locs[i]=="SE33") { #add alleles provided in references
    avNEW = c(5.3,28) #add these
    av = c(av,avNEW)
    fv = c(fv,rep(minF,length(avNEW))) #add freqs
  }
  popFreq[[i]] <-fv
  names(popFreq[[i]]) <-av
 }
 names(popFreq) <- locs
 popFreq = sapply(popFreq,function(x) x/sum(x)) #all alleles must sum to one
 Glist <- euroformix::getGlist(popFreq)
 nL = length(ln) #total number of markers in output

 #create matrix for ref
 bigref = matrix(NA,ncol=3,nrow=M*nL)
 colnames(bigref) <- c("SampleName","Markers","Alleles")
 meta = NULL
 
 for(loc in ln) {
  print(loc)
  #loc =locs[1]
   ins <- (0:(M-1))*nL + which(loc==ln)
   bigref[ins,1] <- rn
   bigref[ins,2] <- loc 
   loc2 = toupper(loc)
   if(loc2%in%locs) { 
    tmp = Glist[[loc2]]$G 
    bigref[ins,3] <- sample( paste0(tmp[,1],"/",tmp[,2]) ,M,Glist[[loc2]]$Gprob, replace=TRUE) #simulated directly
   } else {
    bigref[ins,3] <- rep("",M) #insert empty
   }
 }
} #end if simulated references

 return(list(mix=bigevid,ref=bigref,markers=ln,meta=meta)) 
} #end importData function
